@@ -679,7 +679,7 @@ List simJointTemplate(
     }
     else if(PMFs.size() > 0)
     {
-      if(PMFs.size() != cor.n_cols)
+      if(PMFs.size() - cor.n_cols != 0)
       {
         Rcerr << "`PMFs` and `cor` have different numbers of columns. Quit.\n";
         return List::create();
@@ -1108,6 +1108,571 @@ List SJspearmanPMF(
 }
 
 
+
+
+template<typename valtype>
+inline bool sameSign(valtype x, valtype y)
+{ return (x < 0) == (y < 0); }
+
+
+struct mem
+{
+  double val, *place;
+  mem(){}
+  mem(double val, double *place): val(val), place(place) {}
+};
+
+
+struct twoCor
+{
+  double cor, targetCor;
+  twoCor(){}
+  twoCor(double cor, double targetCor): cor(cor), targetCor(targetCor) {}
+};
+
+
+struct corIndDiff
+{
+  int I, J;
+  double corDiff;
+  // bool operator < (const corIndDiff &X) const
+  // {
+  //   return absCorDiff > X.absCorDiff;
+  // }
+  corIndDiff(){}
+  corIndDiff(int I, int J, double corDiff):
+    I(I), J(J), corDiff(corDiff) {}
+};
+
+
+inline void push(int I, int J, double corDiff,
+                 corIndDiff *v, int size)
+{
+  if(std::abs(corDiff) < std::abs(v[0].corDiff)) return;
+  v[0].corDiff = corDiff;
+  v[0].I = I;
+  v[0].J = J;
+  for(int i = 1; i < size; ++i)
+  {
+    if(std::abs(v[i].corDiff) >=
+       std::abs(v[i - 1].corDiff)) return;
+    std::swap(v[i], v[i - 1]);
+  }
+}
+
+
+
+
+/*
+// [[Rcpp::export]]
+void postSimOpt1(arma::mat &X, arma::mat &Xcor, arma::mat &targetCor,
+                 IntegerVector seed = 123,
+                 int convergenceTail = 10000)
+{
+  INT N = X.n_rows, K = X.n_cols;
+  vec<double> shift, multiplier;
+  normalize<double> (&X[0], N, K, shift, multiplier);
+  if(Xcor.n_cols - K != 0)
+  {
+    Rcout << "`Xcor` has wrong number of columns. Quit.\n";
+    return;
+  }
+  if(targetCor.n_cols - K != 0)
+  {
+    Rcout << "`targetCor` has wrong number of columns. Quit.\n";
+    return;
+  }
+
+
+  RNG rengine, *rng = &rengine;
+  if(seed.size() == 1) rng->seed(seed[0]);
+  else
+  {
+    if(seed.size() * sizeof(int) != sizeof(RNG))
+    {
+      Rcerr << "`seed` should be either an integer or an integer vector of size 626. Quit.\n";
+      return;
+    }
+    rng = (RNG*)&seed[0];
+  }
+
+
+  // Find initial worst entries in correlation matrix.
+  // Calculate initial cost.
+  vec<twoCor> sigma(K * K);
+  vec<twoCor*> sigmacol(K);
+  for(INT i = 0; i < K; ++i) sigmacol[i] = &sigma[0] + K * i;
+  twoCor **Sigma = &sigmacol[0];
+
+
+  for(INT i = 0, iend = K - 1; i < iend; ++i)
+  {
+    for(INT j = i + 1; j < K; ++j)
+    {
+      Sigma[i][j].cor = Xcor.at(j, i);
+      Sigma[i][j].targetCor = targetCor.at(j, i);
+    }
+  }
+
+
+  INT gap = 0;
+  std::uniform_int_distribution<INT> U(0, N - 1);
+  std::bernoulli_distribution B(0.5);
+  // std::uniform_real_distribution<double> unif(0, 1);
+  vec<mem> wouldBeCor(K - 1);
+  double *x = &X[0];
+
+
+  // vec<double> prob(acceptProb.begin(), acceptProb.end());
+  // std::reverse(prob.begin(), prob.end());
+  // std::partial_sum(prob.begin(), prob.end(), prob.begin());
+  // prob.back() = 1;
+  // vec<corIndDiff> nnCandidates(prob.size());
+  // int nn = nnCandidates.size();
+
+
+  while(gap < convergenceTail)
+  {
+    double worstErr = 0;
+    INT I = 0, J = 0;
+    // for(int i = 0; i < nn; ++i) nnCandidates[i].corDiff = 0;
+    for(int i = 0, iend = K - 1; i < iend; ++i)
+    {
+      for(int j = i + 1, jend = K; j < jend; ++j)
+      {
+        double tmp = Sigma[i][j].cor - Sigma[i][j].targetCor;
+        if(std::abs(tmp) <= std::abs(worstErr)) continue;
+        worstErr = tmp;
+        I = i;
+        J = j;
+        // push(i, j, Sigma[i][j].cor - Sigma[i][j].targetCor,
+        //      &nnCandidates[0], nn);
+      }
+    }
+
+
+    // int chosen = std::lower_bound(
+    //   prob.begin(), prob.end(), unif(*rng)) - prob.begin();
+    // I = nnCandidates[chosen].I;
+    // J = nnCandidates[chosen].J;
+
+
+    // INT col = I;
+    // if(B(rng)) col = J;
+    // If we swap x[col][u], x[col][v], what would happen...?
+    // First see
+    double *P = x + I * N, *Q = x + J * N;
+    INT u, v;
+    bool negativeWorstErr = worstErr < 0;
+    while(true)
+    {
+      u = U(*rng); v = U(*rng);
+      while(u == v) {u = U(*rng); v = U(*rng);}
+      // double pdiff = , qdiff = ;
+      bool haveSameSign = sameSign(P[u] - P[v], Q[u] - Q[v]);
+      // if((negativeWorstErr and !haveSameSign) or
+      //    (!negativeWorstErr and haveSameSign)) break;
+      if(negativeWorstErr != haveSameSign) break;
+    }
+
+
+    // u, v can reduce the error in the worst entry of correlation matrix,
+    // but do they also reduce the cost function value?
+    // double *col = I * N + x;
+    INT col = I;
+    if(B(*rng)) col = J; // swap() elements would apply to this column.
+    double costDiff = 0;
+    double &colu = X.at(u, col), &colv = X.at(v, col);
+    wouldBeCor.resize(0);
+    for(INT i = 0; i < K; ++i)
+    {
+      twoCor *a = nullptr;
+      if(i < col) a = &Sigma[i][col];
+      else if (i > col) a = &Sigma[col][i];
+      else continue;
+      double iu = X.at(u, i), iv = X.at(v, i);
+      // double currentInnerProd = iu * colu + iv * colv;
+      // double tentativeInnerProd = iu * colv + iv * colu;
+      // double tentativeCor = a->cor + (tentativeInnerProd - currentInnerProd);
+      double tentativeCor = a->cor + (iu - iv) * (colv - colu);
+      costDiff += std::pow(tentativeCor - a->targetCor, 2) -
+        std::pow(a->cor - a->targetCor, 2);
+      wouldBeCor.push_back(mem(tentativeCor, &a->cor));
+    }
+
+
+    if(costDiff < 0)
+    {
+      gap = 0;
+      std::swap(colu, colv);
+      for(INT i = 0, iend = wouldBeCor.size(); i < iend; ++i)
+        *wouldBeCor[i].place = wouldBeCor[i].val;
+    }
+    else ++gap;
+  }
+
+
+  recover<double> (&X[0], N, K, shift, multiplier);
+  for(INT i = 0, iend = K - 1; i < iend; ++i)
+  {
+    for(INT j = i + 1; j < K; ++j)
+    {
+      Xcor.at(j, i) = Sigma[i][j].cor;
+      Xcor.at(i, j) = Sigma[i][j].cor;
+    }
+  }
+}
+
+
+
+
+// [[Rcpp::export]]
+void postSimOpt2(arma::mat &X, arma::mat &Xcor, arma::mat &targetCor,
+                 IntegerVector seed = 123,
+                 int convergenceTail = 10000)
+{
+  INT N = X.n_rows, K = X.n_cols;
+  vec<double> shift, multiplier;
+  normalize<double> (&X[0], N, K, shift, multiplier);
+  if(Xcor.n_cols - K != 0)
+  {
+    Rcout << "`Xcor` has wrong number of columns. Quit.\n";
+    return;
+  }
+  if(targetCor.n_cols - K != 0)
+  {
+    Rcout << "`targetCor` has wrong number of columns. Quit.\n";
+    return;
+  }
+
+
+  RNG rengine, *rng = &rengine;
+  if(seed.size() == 1) rng->seed(seed[0]);
+  else
+  {
+    if(seed.size() * sizeof(int) != sizeof(RNG))
+    {
+      Rcerr << "`seed` should be either an integer or an integer vector of size 626. Quit.\n";
+      return;
+    }
+    rng = (RNG*)&seed[0];
+  }
+
+
+  // Find initial worst entries in correlation matrix.
+  // Calculate initial cost.
+  vec<twoCor> sigma(K * K);
+  vec<twoCor*> sigmacol(K);
+  for(INT i = 0; i < K; ++i) sigmacol[i] = &sigma[0] + K * i;
+  twoCor **Sigma = &sigmacol[0];
+
+
+  for(INT i = 0, iend = K - 1; i < iend; ++i)
+  {
+    for(INT j = i + 1; j < K; ++j)
+    {
+      Sigma[i][j].cor = Xcor.at(j, i);
+      Sigma[i][j].targetCor = targetCor.at(j, i);
+    }
+  }
+
+
+  INT gap = 0;
+  std::uniform_int_distribution<INT> U(0, N - 1);
+  std::uniform_int_distribution<INT> Uk(0, K - 1);
+  // std::bernoulli_distribution B(0.5);
+  vec<mem> wouldBeCor(K - 1);
+  double *x = &X[0];
+
+
+  while(gap < convergenceTail)
+  {
+
+    INT I = 0, J = 0;
+    // for(int i = 0, iend = K - 1; i < iend; ++i)
+    // {
+    //   for(int j = i + 1, jend = K; j < jend; ++j)
+    //   {
+    //     double tmp = Sigma[i][j].cor - Sigma[i][j].targetCor;
+    //     if(std::abs(tmp) <= std::abs(worstErr)) continue;
+    //     worstErr = tmp;
+    //     I = i;
+    //     J = j;
+    //   }
+    // }
+    while(I == J) { I = Uk(*rng); J = Uk(*rng); }
+    double worstErr = Sigma[I][J].cor - Sigma[I][J].targetCor;
+
+
+    double *P = x + I * N, *Q = x + J * N;
+    INT u, v;
+    bool negativeWorstErr = worstErr < 0;
+    while(true)
+    {
+      // u = U(*rng); v = U(*rng);
+      u = v = 0;
+      while(u == v) {u = U(*rng); v = U(*rng);}
+      bool haveSameSign = sameSign(P[u] - P[v], Q[u] - Q[v]);
+      if(negativeWorstErr != haveSameSign) break;
+    }
+
+
+    INT col = I;
+    // if(B(*rng)) col = J;
+    double costDiff = 0;
+    double &colu = X.at(u, col), &colv = X.at(v, col);
+    wouldBeCor.resize(0);
+    for(INT i = 0; i < K; ++i)
+    {
+      twoCor *a = nullptr;
+      if(i < col) a = &Sigma[i][col];
+      else if (i > col) a = &Sigma[col][i];
+      else continue;
+      double iu = X.at(u, i), iv = X.at(v, i);
+      double tentativeCor = a->cor + (iu - iv) * (colv - colu);
+      costDiff += std::pow(tentativeCor - a->targetCor, 2) -
+        std::pow(a->cor - a->targetCor, 2);
+      wouldBeCor.push_back(mem(tentativeCor, &a->cor));
+    }
+
+
+    if(costDiff < 0)
+    {
+      gap = 0;
+      std::swap(colu, colv);
+      for(INT i = 0, iend = wouldBeCor.size(); i < iend; ++i)
+        *wouldBeCor[i].place = wouldBeCor[i].val;
+    }
+    else ++gap;
+  }
+
+
+  recover<double> (&X[0], N, K, shift, multiplier);
+  for(INT i = 0, iend = K - 1; i < iend; ++i)
+  {
+    for(INT j = i + 1; j < K; ++j)
+    {
+      Xcor.at(j, i) = Sigma[i][j].cor;
+      Xcor.at(i, j) = Sigma[i][j].cor;
+    }
+  }
+}
+*/
+
+
+
+
+// [[Rcpp::export]]
+List postSimOpt(NumericMatrix X,
+                NumericMatrix cor,
+                NumericMatrix Xcor = NumericMatrix(),
+                NumericVector acceptProb = 1.0,
+                IntegerVector seed = 123,
+                int convergenceTail = 10000)
+{
+  INT N = X.nrow(), K = X.ncol();
+  NumericMatrix X_(N, K);
+  std::copy(X.begin(), X.end(), X_.begin());
+
+
+  vec<double*> xptr(K);
+  for(INT i = 0; i < K; ++i) xptr[i] = &X_[0] + i * N;
+  double **x = &xptr[0];
+
+
+  vec<double> shift, multiplier;
+  normalize<double> (&x[0][0], N, K, shift, multiplier);
+  if(Xcor.ncol() - K != 0 and Xcor.size() > 1)
+  {
+    Rcout << "`Xcor` has wrong number of columns. Quit.\n";
+    return List::create();
+  }
+  if(cor.ncol() - K != 0)
+  {
+    Rcout << "`cor` has wrong number of columns. Quit.\n";
+    return List::create();
+  }
+
+
+  RNG rengine, *rng = &rengine;
+  if(seed.size() == 1) rng->seed(seed[0]);
+  else
+  {
+    if(seed.size() * sizeof(int) != sizeof(RNG))
+    {
+      Rcerr << "`seed` should be either an integer or an integer vector of size 626. Quit.\n";
+      return List::create();
+    }
+    rng = (RNG*)&seed[0];
+  }
+
+
+  // Find initial worst entries in correlation matrix.
+  vec<twoCor> sigma(K * K);
+  vec<twoCor*> sigmacol(K);
+  for(INT i = 0; i < K; ++i) sigmacol[i] = &sigma[0] + K * i;
+  twoCor **Sigma = &sigmacol[0];
+
+
+
+
+  // If the current correlations are not given, calculate them.
+  if(Xcor.size() > 1)
+  {
+    for(INT i = 0, iend = K - 1; i < iend; ++i)
+    {
+      double *XcorV = &Xcor[0] + i * K;
+      double *targetCorV = &cor[0] + i * K;
+      for(INT j = i + 1; j < K; ++j)
+      {
+        Sigma[i][j].cor = XcorV[j];
+        Sigma[i][j].targetCor = targetCorV[j];
+      }
+    }
+  }
+  else
+  {
+    for(INT i = 0, iend = K - 1; i < iend; ++i)
+    {
+      double *targetCorV = &cor[0] + i * K;
+      for(INT j = i + 1; j < K; ++j)
+      {
+        Sigma[i][j].cor = std::inner_product(
+          &x[i][0], &x[i][0] + N, &x[j][0], 0.0);
+        Sigma[i][j].targetCor = targetCorV[j];
+      }
+    }
+  }
+
+
+
+
+  INT gap = 0;
+  std::uniform_int_distribution<INT> U(0, N - 1);
+  std::bernoulli_distribution B(0.5);
+  std::uniform_real_distribution<double> unif(0, 1);
+  vec<mem> wouldBeCor(K - 1);
+
+
+  vec<double> prob(acceptProb.begin(), acceptProb.end());
+  std::reverse(prob.begin(), prob.end());
+  std::partial_sum(prob.begin(), prob.end(), prob.begin());
+  prob.back() = 1;
+  vec<corIndDiff> nnCandidates(prob.size());
+  int nn = nnCandidates.size();
+
+
+  while(gap < convergenceTail)
+  {
+    double worstErr = 0;
+    INT I = 0, J = 0;
+    for(int i = 0; i < nn; ++i) nnCandidates[i].corDiff = 0;
+    for(int i = 0, iend = K - 1; i < iend; ++i)
+    {
+      for(int j = i + 1, jend = K; j < jend; ++j)
+      {
+        push(i, j, Sigma[i][j].cor - Sigma[i][j].targetCor,
+             &nnCandidates[0], nn);
+      }
+    }
+
+
+    int chosen = std::lower_bound(
+      prob.begin(), prob.end(), unif(*rng)) - prob.begin();
+    I = nnCandidates[chosen].I;
+    J = nnCandidates[chosen].J;
+    worstErr = nnCandidates[chosen].corDiff;
+
+
+    double *P = x[I], *Q = x[J];
+    INT u, v;
+    bool negativeWorstErr = worstErr < 0;
+    while(true)
+    {
+      u = U(*rng); v = U(*rng);
+      while(u == v) {u = U(*rng); v = U(*rng);}
+      bool haveSameSign = sameSign(P[u] - P[v], Q[u] - Q[v]);
+      if(negativeWorstErr != haveSameSign) break;
+    }
+
+
+    // u, v can reduce the error in the worst entry of correlation matrix,
+    // but do they also reduce the cost function value?
+    // double *col = I * N + x;
+    INT col = I;
+    if(B(*rng)) col = J; // swap() elements would apply to this column.
+    double costDiff = 0;
+    // double &colu = X_(u, col), &colv = X_(v, col);
+    double &colu = x[col][u], &colv = x[col][v];
+    wouldBeCor.resize(0);
+    for(INT i = 0; i < K; ++i)
+    {
+      twoCor *a = nullptr;
+      if(i < col) a = &Sigma[i][col];
+      else if (i > col) a = &Sigma[col][i];
+      else continue;
+      double iu = x[i][u], iv = x[i][v];
+      double tentativeCor = a->cor + (iu - iv) * (colv - colu);
+      costDiff += std::pow(tentativeCor - a->targetCor, 2) -
+        std::pow(a->cor - a->targetCor, 2);
+      wouldBeCor.push_back(mem(tentativeCor, &a->cor));
+    }
+
+
+    if(costDiff < 0)
+    {
+      gap = 0;
+      std::swap(colu, colv);
+      for(INT i = 0, iend = wouldBeCor.size(); i < iend; ++i)
+        *wouldBeCor[i].place = wouldBeCor[i].val;
+    }
+    else ++gap;
+  }
+
+
+  NumericMatrix corNew(K, K);
+  for(INT i = 0; i < K; ++i)
+  {
+    corNew(i, i) = 1;
+    for(INT j = i + 1; j < K; ++j)
+    {
+      corNew(j, i) = Sigma[i][j].cor;
+      corNew(i, j) = Sigma[i][j].cor;
+    }
+  }
+  return List::create(Named("X") = X_, Named("cor") = corNew);
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
